package oracle.toplink.testing.tests.unitofwork.writechanges;

import java.math.BigDecimal;

import oracle.toplink.expressions.Expression;
import oracle.toplink.expressions.ExpressionBuilder;
import oracle.toplink.internal.databaseaccess.DatasourceAccessor;
import oracle.toplink.queryframework.ReadObjectQuery;
import oracle.toplink.sessions.UnitOfWork;
import oracle.toplink.testing.framework.AutoVerifyTestCase;
import oracle.toplink.testing.framework.TestErrorException;
import oracle.toplink.testing.models.employee.domain.Employee;

/**
 *  @author  xiaosche
 *  The test verifies whether the number executed statements being caculated properly. The test is prepared for
 *  bug 5888543 and 6022960.
 */
public class WriteChangesFailed_StatementCountTestCase extends AutoVerifyTestCase {
    public BigDecimal id = null;
    public int writeStatementsCount = 0;
    public int readStatementsCount = 0;
    public int storedprocedureStatementsCount = 0;
    public int originalReadStatementsCount = 0;
    public int originalWriteStatementsCount = 0;
    public int originalStoredProcedureStatementsCount = 0;


    public void setup() {
        id = new BigDecimal(999);
        UnitOfWork uow = getSession().acquireUnitOfWork();


        Employee employee = new Employee();
        employee = (Employee)uow.registerObject(employee);
        employee.setId(id);
        employee.setFirstName("foo");
        uow.commit();
    }

    public void test() {
        UnitOfWork uow = getSession().acquireUnitOfWork();
        originalWriteStatementsCount = ((DatasourceAccessor)uow.getParent().getAccessor()).writeStatementsCount;
        originalReadStatementsCount = ((DatasourceAccessor)uow.getParent().getAccessor()).readStatementsCount;
        originalStoredProcedureStatementsCount = 
                ((DatasourceAccessor)uow.getParent().getAccessor()).storedProcedureStatementsCount;

        try {
            Employee employee = new Employee();
            employee = (Employee)uow.registerNewObject(employee);
            employee.setId(id);
            employee.setFirstName("foo-modified");

            try {
                uow.writeChanges();
            } catch (Exception e) {
                if (uow.getProperties() != null) {
                    if (uow.getProperties().get(DatasourceAccessor.READ_STATEMENTS_COUNT_PROPERTY) != null) {
                        readStatementsCount = 
                                ((Integer)uow.getProperties().get(DatasourceAccessor.READ_STATEMENTS_COUNT_PROPERTY)).intValue();
                    }
                    if (uow.getProperties().get(DatasourceAccessor.WRITE_STATEMENTS_COUNT_PROPERTY) != null) {
                        writeStatementsCount = 
                                ((Integer)uow.getProperties().get(DatasourceAccessor.WRITE_STATEMENTS_COUNT_PROPERTY)).intValue();
                    }
                    if (uow.getProperties().get(DatasourceAccessor.STOREDPROCEDURE_STATEMENTS_COUNT_PROPERTY) != 
                        null) {
                        storedprocedureStatementsCount = 
                                ((Integer)uow.getProperties().get(DatasourceAccessor.STOREDPROCEDURE_STATEMENTS_COUNT_PROPERTY)).intValue();
                    }
                }
            }

        } finally {
            uow.release();
        }
    }

    public void verify() {
        if ((readStatementsCount - originalReadStatementsCount) != 0) {
            throw new TestErrorException("The desirable number of read statement is 0, it however return:" + 
                                         (readStatementsCount - originalReadStatementsCount));
        }
        if ((writeStatementsCount - originalWriteStatementsCount) != 1) {
            throw new TestErrorException("The desirable number of write statement is 1, it however return:" + 
                                         (writeStatementsCount - originalWriteStatementsCount));
        }
        if ((storedprocedureStatementsCount - originalStoredProcedureStatementsCount) != 0) {
            throw new TestErrorException("The desirable number of write statement is 0, it however return:" + 
                                         (storedprocedureStatementsCount - 
                                          originalStoredProcedureStatementsCount));
        }
    }

    public void reset() {
        if (id == null) {
            return;
        }
        UnitOfWork uow = getSession().acquireUnitOfWork();

        Expression expression = (new ExpressionBuilder()).get("id").equal(id);
        ReadObjectQuery query = new ReadObjectQuery(Employee.class, expression);

        Employee employee = (Employee)uow.executeQuery(query);

        uow.deleteObject(employee);
        uow.commit();
    }

    protected void resetVerify() {
        getSession().getIdentityMapAccessor().initializeAllIdentityMaps();
        Expression expression = (new ExpressionBuilder()).get("id").equal(id);
        ReadObjectQuery query = new ReadObjectQuery(Employee.class, expression);

        Employee employee = (Employee)getSession().executeQuery(query);

        id = null;
        if (employee != null) {
            throw new TestErrorException("Employee not removed after the test.");
        }
    }
}
